package org.hardknockjava.lesson09.exercise02;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;

public class Lesson09 {

    public static void main(String[] args) throws IOException {
        Lesson09 lesson = new Lesson09();
        String fileName = "lesson09-input.txt";

        String inputDirectoryName = "./input";
        File inputFile = new File(inputDirectoryName, fileName);

        System.out.println();
        System.out.println("The input file name is " + inputFile.getCanonicalPath());
        System.out.println();

        String outputDirectoryName = "./output";
        File outputDirectory = new File(outputDirectoryName);
        outputDirectory.mkdirs();

        try {
            lesson.displayFileBuffered(inputDirectoryName, inputFile, outputDirectory,
                    "lesson09-output-buffered.txt");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // This method uses a buffered reader and writer.
    // Notice that we treat the input and output as lines
    // of text that don't include end-of-line characters.
    // Because of that, we use the newline() method to
    // terminate each output line.
    private void displayFileBuffered(String directoryName, File inputFile, File outputDirectory,
            String outputFileName) throws IOException {
        System.out.println("This is the buffered version.");

        File outputFile = new File(outputDirectory, outputFileName);
        deleteOutputFile(outputFile);

        {
            Reader inRdr = new FileReader(inputFile);
            BufferedReader bufRdr = new BufferedReader(inRdr);
            
            Writer outWtr = new FileWriter(outputFile);
            BufferedWriter bufWtr = new BufferedWriter(outWtr);

            String buffer = "";

            try {
                while ((buffer = bufRdr.readLine()) != null) {
                    bufWtr.write(buffer);
                    bufWtr.newLine();
                    bufWtr.flush();
                }
            } finally {
                bufRdr.close();
                bufWtr.close();
                inRdr.close();
                outWtr.close();
            }
        }

        {
            Reader outRdr = new FileReader(outputFile);
            BufferedReader bufRdr = new BufferedReader(outRdr);
            String buffer = "";

            try {
                while ((buffer = bufRdr.readLine()) != null) {
                    System.out.println(buffer);
                }
            } finally {
                outRdr.close();
                System.out.println("\n\n******************************\n\n");
            }
        }
    }

    private void deleteOutputFile(File outputFile) {
        System.out.println();
        try {
            if (outputFile.delete()) {
                System.out.println("The output file " + outputFile.getCanonicalPath() + " was deleted.");
            } else {
                System.out.println("The output file " + outputFile.getCanonicalPath() + " was not deleted.");
            }
        } catch (IOException e) {
            System.out.println("An exception occurred when trying to delete the output file.");
        }
        System.out.println();
    }
}
